#include <math.h>


typedef int boolean;
#define TRUE (0==0)
#define FALSE (0!=0)

#ifdef __NVCC__
#define __BOTH__ __device__ __host__
#else
#define __BOTH__ /**/
#endif

#define NUM_THREADS 1024

#include <ruby.h>

// TODO: this doesn't work with nvcc!
#ifdef DEBUG
#define debug(format, ...)				\
  fprintf(stderr, "DEBUG: " format "\n", ##__VA_ARGS__)
#else
#define debug(format, ...) while(0)
#endif
// A memo for an infrequent C programmer: CPP concatenates string
// literals in juxtapositions; so "DEBUG: " format "\n" will become
// one string constant.  __VA_ARGS__ are the variable length arguments
// after format.  The preceding ## will erase the comma after "\n"
// when there is no argument after format.

// init_parray : void -> void

// usage: init_parray creates a Ruby class PArray and registeres
// alloc_int_array and free array in the class.  If the class was
// already defined, it adds those methods into the existing class.
extern "C" void Init_parray_c();


// get_parray : RubyObjct -> int*

// usage: get_parray(wrapped_ptr) extracts a wrapped pointer in the
// given wrapped_ptr Ruby object.
extern "C" void* unwrap_pointer(VALUE wrapped_ptr);

#define get_parray(wrapped_ptr, element_type) \
  (element_type*)(unwrap_pointer(wrapped_ptr))

// IKRA_NUM2FLOAT : RubyObject -> float

// usage: IKRA_NUM2FLOAT(wrapped_number) converts a Ruby Number object into
// float (i.e., 32bit floating point) value in C.  

#define IKRA_NUM2FLOAT(x) ((float)(NUM2DBL(x)))

#define safe_aref1(ptr,index0,size0,pad) \
  ((0<=(index0) && (index0)<(size0))?(ptr)[index0]:(pad))

#define safe_aref2(ptr,index0,index1,size0,size1,pad)	\
  ((0<=(index0) && (index0)<(size0) \
    && 0<=(index1) && (index1)<(size1))\
   ?(ptr)[(index0)*(size0)+(index1)]:(pad))

#define safe_aref3(ptr,index0,index1,index2,size0,size1,size2,pad)	\
  ((0<=(index0) && (index0)<(size0) \
    && 0<=(index1) && (index1)<(size1)\
    && 0<=(index2) && (index2)<(size2))\
   ?(ptr)[((index0)*(size0)+(index1))*(size1)+(index2)]:(pad))

__BOTH__
inline float fexp(float x, float y) {
  return exp(y * log(x));
}

// to represent range objects in Ruby, which are constructed by n..m.
typedef struct _Ikra_range {
  int min, max;
  __BOTH__
  _Ikra_range(): min(0), max(0) {}
  __BOTH__
  _Ikra_range(int _min, int _max): min(_min), max(_max) {}
} ikra_range_t;

#ifdef __NVCC__
// Wrappers of CUDA runtime APIs that checks errors. 
#include <helper_cuda.h>
// memory allocation on the device memory
#define ikra_alloc(ptr, size)  checkCudaErrors(cudaMalloc(ptr,size))
// memory copy (cudaMemcpy)
#define ikra_memcpy(dst,src,size,direction) \
  checkCudaErrors(cudaMemcpy(dst, src, size, direction))
// memory deallocation
#define ikra_free(ptr)	checkCudaErrors(cudaFree(ptr))

// error check after each kernel function call
#define ikra_check_kernel_call() \
  checkCudaErrors(cudaPeekAtLastError())
#endif
