/* This file is a prototype of an Ikra generated CUDA program.

TODO: support multiple architectures;
      CUDA version's alloc and free should operate on the GPU device memory

*/

/* to provide Ruby wrappers */
#include <stdio.h>
#include <ikra.h>

// alloc_int_array : RubyObject * RubyInt -> RubyObject 

// usage: alloc_int_array(ignore_self,length) -> ptr allocates a
// memory block for an integer array of the given length, and returns
// the address of the array as a Ruby object.
VALUE rb_alloc_int_array(VALUE __self, VALUE __length, VALUE __bytes) {

  int length = NUM2INT(__length);

  int* bytes;
  if (TYPE(__bytes) == T_STRING) {
    bytes = (int*)RSTRING_PTR(__bytes);
  } else if (!RTEST(__bytes)) {//__bytes is nil or false
    bytes = NULL;
  } else {
    debug("alloc_int_array requiers either nil or a string");
    return Qnil;
  }

  int* ptr;
#ifdef __NVCC__
  // NOTE: CUDA version allocates GPU device memory (and returns
  // nilwhen it cannot allocate)
  if (cudaMalloc((void**) &ptr, sizeof(int)*length)) {
    debug("alloc_int_array(%d) failed", (unsigned int)length);
    return Qnil;
  }
  if (bytes!=NULL)
    ikra_memcpy(ptr, bytes, length*sizeof(int), cudaMemcpyHostToDevice);
#else
  // ALLOC_N is a wrapper of malloc in Ruby but ensures successful
  // allocation.
  ptr =    ALLOC_N(int, length); 
  if (bytes!=NULL)
    memcpy(ptr,bytes,length*sizeof(int));
#endif

  // Data_Wrap_Struct(rb_cObject, 0, 0, ptr) allocates a Ruby object
  // of the rb_cObject class that wraps ptr.
  VALUE obj = Data_Wrap_Struct(rb_cObject, 0, 0, ptr);
  debug("alloc_int_array(%d) -> 0x%p wrapped in 0x%p",
	(unsigned int)length, ptr, (void**) obj);
  return obj;
}

// get_parray : RubyObjct -> int*

// usage: get_parray(wrapped_ptr) extracts a wrapped pointer in the
// given wrapped_ptr Ruby object.
void* unwrap_pointer(VALUE wrapped_ptr) {
  void *ptr;
  // Data_Get_Struct(wrapped_ptr, int, ptr) extracts a wrapped pointer
  // to integers from wrapped_ptr.
  Data_Get_Struct(wrapped_ptr, void, ptr);
  debug("get_parray(0x%p) -> 0x%p", (void**)wrapped_ptr, ptr);
  return ptr;
}

// rb_free_array : RubyObject * RubyObject -> RubyObject

// usage: rb_free_array(_,wrapped_ptr) frees the malloc'd pointer
// wrapped in the given wrapped_ptr.
VALUE rb_free_array(VALUE self, VALUE array) {

  int *ptr = get_parray(array,int);
  debug("free_malloced_memory(0x%p)", ptr);

#ifdef __NVCC__
  // NOTE: CUDA version should use CUDA's free function.
  cudaFree(ptr);
  // TODO: check error
#else
  xfree(ptr); // Ruby's free.
#endif
  return Qnil;
}

// rb_int_array2str : RubyObject * RubyObject * RubyObject -> RubyObject

// usage: rb_int_array2str(_,ptr,size) return the body of the array as
// a Ruby string.  This is a handy way to pass binary data to some
// other native library.

VALUE rb_int_array2str(VALUE __self, VALUE __ptr, VALUE __length) {
  char* ptr = (char*)unwrap_pointer(__ptr);
  int length = NUM2INT(__length);
  char* host_ptr;
#ifdef __NVCC__
  // NOTE: CUDA version allocates a copy area in the host memory, and
  // then copy the contents from the device memory.
  host_ptr = (char*)ALLOCA_N(int, length);
  ikra_memcpy(host_ptr, ptr, length*sizeof(int), cudaMemcpyDeviceToHost);
  // TODO: check error
#else
  host_ptr = ptr;
#endif
  return rb_str_new(host_ptr, length*sizeof(int));
}

// init_parray : void -> void

// usage: init_parray creates a Ruby class PArray and registeres
// alloc_int_array and free array in the class.  If the class was
// already defined, it adds those methods into the existing class.
extern "C"
void INIT_FUNCTION_NAME() //defined by the Makefile
{
  VALUE cPArray = rb_define_class("PArray", rb_cObject);
  rb_define_singleton_method(cPArray, "alloc_int_array",
			     (VALUE(*)(...))rb_alloc_int_array, 2);
  rb_define_singleton_method(cPArray, "free_array",
			     (VALUE(*)(...))rb_free_array, 1);
  rb_define_singleton_method(cPArray, "int_array2str",
			     (VALUE(*)(...))rb_int_array2str, 2);
  return;
}
