module type CONVERT_KERNEL =
  sig
    type kind = Initialize | Map | Reduce | UnitlessReduce
    val string_of_kind : kind -> string
    val element_type_of : Ikra_type.t -> Ikra_type.t
    type kernel = {
      kind : kind;
      name : string;
      free_vars : Ast.tast list;
      params : Ast.parameter_pattern;
      type_decls : Ast.type_decl list;
      body : Ast.tast list;
      body_type : Ikra_type.t;
      receiver : Ast.tast;
      top_decls : Ast.tast;
      return_type : Ikra_type.t;
    }
    val top_decls : kernel list -> Ast.tast list
    type var_iname_type_cname = Ast.tast * string
    type env = {
      wrapper_name : string;
      kernel_fun_name : string;
      body_fun_name : string;
      argspecs : var_iname_type_cname list;
      body_ctype : Cuda.cuda_type;
      fvspecs : (Ast.tast * string) list;
      body_args : (Cuda.cuda_type * Cuda.variable) list;
      element_statements : Cuda.statement list;
    }
    val generate_makefile : generate_only:bool -> Convert_env.t -> unit
    val cuda_code_of : string -> kernel list -> string

    val cxx_suffix : string 
    val backend_name : string
  end
module M :
  functor (Convert_kernel : CONVERT_KERNEL) ->
    sig
      type tast = Ast.tast
      type variable = Ast.variable
      (* val tint : Ikra_type.t *)
      (* val tobject : Ikra_type.t *)
      (* val target_ruby_suffix : string *)
      (* val target_cuda_suffix : string *)
      type array_ops = Convert_kernel.kernel
      (* val maybe : ('a -> 'b) -> 'a option -> 'b option *)
      (* val is_of_array_type : variable -> Ikra_type.t Env.env -> bool *)
      (* val dup : int -> string -> 'a -> (string * 'a) list *)
      (* val array_op : *)
      (*   Ikra_type.t Env.env -> *)
      (*   tast -> *)
      (*   (Convert_kernel.kind * (string * Ikra_type.t) list * string * tast) *)
      (*   option *)
      (* val expand_tuples : *)
      (*   (string * Ikra_type.t) list -> (string * Ikra_type.t) list *)
      (* val replace_array_operations : string -> tast -> tast * array_ops list *)
      val generate_ruby_file : Convert_env.t -> tast -> array_ops list
      val generate_cuda_file :
        ?generate_only:bool -> Convert_env.t -> Convert_kernel.kernel list
        -> unit
      val target_file_names :
        ?cleanup:bool -> string option -> string -> (Convert_env.t -> 'a) -> 'a
      val generate_and_compile
        : ?generate_only:bool -> Convert_env.t -> tast -> unit
      val run_converted :
        ?generate_only:bool -> ?redirect:bool -> Convert_env.t -> string list
        -> string

    end
