type cuda_type =
    CUDAint
  | CUDAfloat
  | CUDAbool
  | CUDArange                           (* a pair of integer *)
  | CUDApointer of cuda_type * int list
  | CUDAother of string
  | CUDAfunc of cuda_type * cuda_type list
  | CUDAtuple of cuda_type list
	| CUDAshared of cuda_type
	| CUDAstatic of cuda_type

type variable = string
type vardecl = variable * cuda_type
type unary_op =
    Make_ref
  | Deref
  | Unary_plus
  | Unary_minus
  | Bitwise_not
  | Logical_not
  | Pre_inc
  | Pre_dec
type binary_op =
    Logical_or
  | Logical_and
  | Bitwise_or
  | Exclusive_or
  | Bitwise_and
  | Equal
  | Less_than
  | Greater_than
  | Less_or_equal
  | Greater_or_equal
  | Shift_left
  | Shift_right
  | Add
  | Sub
  | Mul
  | Div
  | Mod
  | Assign
type ternary_op = Cond
type exp =
    Literal of Ast.literal
  | Var of variable
  | Unary_op of unary_op * exp
  | Binary_op of binary_op * exp * exp
  | Ternary_op of ternary_op * exp * exp * exp
  | Statement_exp of statement list * exp
  | Funcall of string * exp list
  | Kernelcall of string * exp * exp * exp list
  | Cast of cuda_type * exp
  | Sizeof of cuda_type
  | Aref of exp * exp
  | Fref of exp * string
and statement =
    Block of statement list
  | While of exp * statement
  | If of exp * statement * statement
  | For of statement * exp * exp * statement
  | Exp of exp
  | Return of exp
  | Return_
  | Vardecl of vardecl
  | Vardecli of vardecl * exp
type fundef = {
  modifier : string;
  return_type : cuda_type;
  name : string;
  formal_params : vardecl list;
  body : statement list;
}
type const_decl = {
  constant_modifier : string;
  constant_type : cuda_type;
  constant_name : string;
  constant_value : exp;
}
type struct_decl = {
  struct_name : string;
  struct_field_types : cuda_type list;
  struct_field_names : string list;
}
type decl =
  | Function_decl of fundef
  | Constant_decl of const_decl
  | Struct_decl of struct_decl

val struct_name_of_cuda_tuple : cuda_type -> string
val constructor_name_of_cuda_tuple : cuda_type -> string

val constant_table : (string * string) list
val is_cxx_defined_constant : string -> bool

val unary_ruby_to_c : (string * unary_op * string) list
val binary_ruby_to_c : (string * binary_op * string) list
val lookup_op : (string * 'a * string) list -> string -> 'a
val lookup_unary_op : string -> unary_op
val method_name_is_unary : string -> bool
val lookup_binary_op : string -> binary_op
val method_name_is_binary : string -> bool
val cuda_type_of_ikra_type : Ikra_type.t -> cuda_type
exception Has_statement
val statement_of_type_decl : Ast.type_decl -> statement
val exp_of_ruby : Ast.tast -> exp
val statement_of_ruby_ast : Ast.tast -> statement
val decl_of_ruby_ast : Ast.tast -> decl
val string_of_cuda_type : ?vname:string -> cuda_type -> string
val string_of_op : ('a * 'b * string) list -> 'b -> string
val string_of_unary : unary_op -> string
val string_of_binary : binary_op -> string
val string_of_exp : exp -> variable
val string_of_statement : statement -> string
val string_of_statements : statement list -> string
val prototype_of_decl : decl -> string
val prepend_modifier : string -> decl -> decl
val string_of_decl : decl -> string
