(* module Helper : *)
(*   sig *)
(*     val make_uniop : Cuda.unary_op -> Cuda.exp -> Cuda.exp *)
(*     val make_binop : Cuda.binary_op -> Cuda.exp -> Cuda.exp -> Cuda.exp *)
(*     val counter : int ref *)
(*     val generate_var_name : 'a -> string *)
(*     val vargen : *)
(*       ?name:Cuda.variable -> Cuda.cuda_type -> Cuda.vardecl * Cuda.exp *)
(*   end *)
val _int : Cuda.cuda_type
val _float : Cuda.cuda_type
(* val _p : Cuda.cuda_type -> Cuda.cuda_type *)
val void : Cuda.cuda_type
val _num : int -> Cuda.exp
val _flt : float -> Cuda.exp
val _str : string -> Cuda.exp
val _ifexp : Cuda.exp -> Cuda.exp -> Cuda.exp -> Cuda.exp
val _statement : Cuda.statement list -> Cuda.exp -> Cuda.exp
val ( **@ ) : string -> Cuda.exp list -> Cuda.exp
val _kernel_call : string -> (Cuda.exp * Cuda.exp) -> Cuda.exp list
  -> Cuda.statement list
val _aref : Cuda.exp -> Cuda.exp -> Cuda.exp
val _cast : Cuda.cuda_type -> Cuda.exp -> Cuda.exp
val sizeof : Cuda.cuda_type -> Cuda.exp
val _fref : Cuda.exp -> string -> Cuda.exp

val _block : Cuda.statement list -> Cuda.statement
val _while : Cuda.exp -> Cuda.statement -> Cuda.statement
val _if : Cuda.exp -> Cuda.statement -> Cuda.statement -> Cuda.statement
val _for :
  Cuda.cuda_type ->
  Cuda.variable ->
  (Cuda.exp -> Cuda.exp * Cuda.exp * Cuda.exp * Cuda.statement) ->
  Cuda.statement
val _exp : Cuda.exp -> Cuda.statement
val return : Cuda.exp -> Cuda.statement
val return_ : Cuda.statement
val _let : Cuda.cuda_type -> Cuda.variable -> Cuda.exp ->
  (Cuda.exp -> Cuda.statement list) -> Cuda.statement list
val _let0 : Cuda.cuda_type -> Cuda.variable -> 
  (Cuda.exp -> Cuda.statement list) -> Cuda.statement list
val defun :
  ?modifier:string ->
  Cuda.cuda_type ->
  string ->
  (Cuda.cuda_type * Cuda.variable) list ->
  (Cuda.exp list -> Cuda.statement list) -> Cuda.fundef
val inline :
  Cuda.cuda_type ->
  string ->
  (Cuda.cuda_type * Cuda.variable) list ->
  (Cuda.exp list -> Cuda.statement list) -> Cuda.fundef
val __global__ :
  Cuda.cuda_type ->
  string ->
  (Cuda.cuda_type * Cuda.variable) list ->
  (Cuda.exp list -> Cuda.statement list) -> Cuda.fundef
val __device__ :
  Cuda.cuda_type ->
  string ->
  (Cuda.cuda_type * Cuda.variable) list ->
  (Cuda.exp list -> Cuda.statement list) -> Cuda.fundef
val __device__inline :
  Cuda.cuda_type ->
  string ->
  (Cuda.cuda_type * Cuda.variable) list ->
  (Cuda.exp list -> Cuda.statement list) -> Cuda.fundef
val __device__host__inline :
	Cuda.cuda_type ->
	string ->
	(Cuda.cuda_type * Cuda.variable) list ->
	(Cuda.exp list -> Cuda.statement list) -> Cuda.fundef
val extern_C :
  Cuda.cuda_type ->
  string ->
  (Cuda.cuda_type * Cuda.variable) list ->
  (Cuda.exp list -> Cuda.statement list) -> Cuda.fundef
val _struct : string ->
  (Cuda.cuda_type * string) list -> 
  Cuda.struct_decl
val _VALUE : Cuda.cuda_type
val _VALUE_func_t : Cuda.cuda_type
val get_parray : Cuda.exp * Cuda.cuda_type -> Cuda.exp
val rb_int_new : Cuda.exp -> Cuda.exp
val rb_float_new : Cuda.exp -> Cuda.exp
val rb_define_module_function :
  Cuda.exp * Cuda.exp * Cuda.exp * Cuda.exp -> Cuda.exp
val rb_define_module : Cuda.exp -> Cuda.exp
val _NUM2INT : Cuda.exp -> Cuda.exp
val _IKRA_NUM2FLOAT : Cuda.exp -> Cuda.exp
val _Qnil : Cuda.exp
val _range_min : Cuda.exp -> Cuda.exp
val _range_max : Cuda.exp -> Cuda.exp
(* val amp : Cuda.exp -> Cuda.exp *)

val _NUM_THREADS : Cuda.exp
val blockIdx_x  : Cuda.exp
val blockDim_x  : Cuda.exp
val threadIdx_x : Cuda.exp
val gridDim_x : Cuda.exp
val blockIdx_y  : Cuda.exp
val blockDim_y  : Cuda.exp
val threadIdx_y : Cuda.exp
val blockIdx_z  : Cuda.exp
val blockDim_z  : Cuda.exp
val threadIdx_z : Cuda.exp
type cudaMemcpyKind =
  | CudaMemcpyDeviceToHost | CudaMemcpyHostToDevice | CudaMemcpyDeviceToDevice
val cudaMemcpy  : Cuda.exp -> Cuda.exp -> Cuda.exp -> cudaMemcpyKind
  -> Cuda.exp
val cudaMalloc  : Cuda.exp -> Cuda.exp -> Cuda.exp
val cudaFree    : Cuda.exp -> Cuda.exp

val ( ?* ) : Cuda.cuda_type -> Cuda.cuda_type

val ( !& ) : Cuda.exp -> Cuda.exp
val ( !* ) : Cuda.exp -> Cuda.exp
val ( !+ ) : Cuda.exp -> Cuda.exp
val ( !- ) : Cuda.exp -> Cuda.exp
val ( !~ ) : Cuda.exp -> Cuda.exp
val ( !! ) : Cuda.exp -> Cuda.exp
val ( !++ ): Cuda.exp -> Cuda.exp
val ( !-- ): Cuda.exp -> Cuda.exp

(* val ( ! ) : Cuda.exp -> Cuda.exp *)
(* val ( ++ ) : Cuda.exp -> Cuda.exp *)
(* val ( -- ) : Cuda.exp -> Cuda.exp *)
val ( ||@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( |@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( &&@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( ^@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( &@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( ==@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( <@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( >@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( <=@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( >=@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( <<@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( >>@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( +@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( -@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( *@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( /@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( %@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
val ( =@ ) : Cuda.exp -> Cuda.exp -> Cuda.exp
