
type variable = string

type 'a env =
  | Empty
  | Frame of 'a env * (variable * 'a) list 

let empty = Empty;;

exception Unbound of string;;
  
let extend : 'a env -> (variable * 'a) list -> 'a env =
  (* to construct an extended environment from the given environment,
     and the alist, i.e., a list of pair of variable name and its
     value *)

  fun parent_env alist -> Frame(parent_env, alist)

let lookup : 'a env -> variable -> 'a =
  
  fun top_env search_var ->
    let rec lookup_in = function
      | Empty -> raise (Unbound search_var)
      | Frame(parent, alist) ->
	try List.assoc search_var alist with
	    Not_found -> lookup_in parent in
    lookup_in top_env


let rec is_bound : 'a env -> variable -> bool =
  (* to determine if the given variable is bound in the given env *)

  fun env search_var ->
    try let _ = lookup env search_var
	in true
    with Unbound _ -> false

    (* | Empty -> false *)
    (* | Frame(parent, alist) -> *)
    (*   (List.exists (fun (v,_) -> v=search_var) alist) || *)
    (* 	(bound parent search_var) *)

let rec bound_vars : 'a env -> variable list =

  function
    | Empty -> []
    | Frame(parent, alist) ->
      (List.map (fun (v,_) -> v) alist) @
	(bound_vars parent)
