open OUnit2

type test_case = {
  title : string;
  expected : string;
  gen : unit -> Cuda.fundef;
}

let tests : test_case list = let open Cuda_constructors in [
  {
    title="__ikra_body_reduce0";
    expected = "\
inline int __ikra_body_reduce0(int x,int y){
    return (x)+(y);
}";
    gen = fun () ->
      inline _int "__ikra_body_reduce0" [_int, "x"; _int, "y"]
	(function [x;y] -> [ return (x +@ y); ] | _-> [])
  };

  {
    title = "__ikra_kernel_reduce0";
    expected = "\
inline int __ikra_kernel_reduce0(int* ptr,int size0,int size1,int zero){
    int accum=zero;
    int size=(size0)*(size1);
    for(int i=0;(i)<(size);++(i))
	(accum)=(__ikra_body_reduce0(accum,ptr[i]));
    return accum;
}";
    gen = fun () ->
      inline _int "__ikra_kernel_reduce0"
        [?* _int, "ptr"; _int, "size0"; _int, "size1"; _int, "zero"]
        (function [ptr; size0; size1; zero] -> 
          _let _int "accum" zero (fun accum ->
            _let _int "size" (size0 *@ size1) (fun size ->
              [ _for _int "i" (fun i ->
                (_num 0, i <@ size, !++ i, 
                 _exp(accum =@ ("__ikra_body_reduce0" **@
				 [accum; _aref ptr i]))));
                return accum;]))
	  |_ -> []);
  };

  {
    title = "__ikra_wrapper_reduce0";
    expected = "\
VALUE __ikra_wrapper_reduce0(VALUE self,VALUE wrapped_array,VALUE _size0,
			     VALUE _size1,VALUE _unit){
    return rb_int_new(
	       __ikra_kernel_reduce0(
	           get_parray(wrapped_array,int),
		   NUM2INT(_size0),
		   NUM2INT(_size1),
		   NUM2INT(_unit)));
}";
    gen = fun () ->
      defun _VALUE "__ikra_wrapper_reduce0"
      [_VALUE, "self"; _VALUE, "wrapped_array"; _VALUE, "_size0";
       _VALUE, "_size1"; _VALUE, "_unit"]
	(function [self; wrapped_array; _size0; _size1; _unit] ->
          [
	    return (rb_int_new
		      ("__ikra_kernel_reduce0" **@
			 [get_parray(wrapped_array,_int);
			  _NUM2INT(_size0);
			  _NUM2INT(_size1);
			  _NUM2INT(_unit);]));
	  ] |_->[])
  };
]

let cmp : string -> string -> bool =
  let newline_matcher = Str.regexp "\n[ \t]*" in
  let erase_newlines = Str.global_replace newline_matcher "" in
  fun s1 s2 -> (erase_newlines s1) = (erase_newlines s2)
      
let test_cases = List.map
  (fun {title; expected; gen} ->
    title >::
      (fun _ ->
        let generated_ast = gen() in
        let generated_str =
          Cuda.string_of_decl (Cuda.Function_decl generated_ast) in
        assert_equal ~printer:(fun x->x) ~cmp expected generated_str))
  tests

let suite = "cuda_constructors_test" >::: test_cases
  
let () =
  run_test_tt_main suite
