open OUnit2

let _ = Config.initialize None

let _ = print_string "preparing data\n"

let test_data : (Ast.tast * Ast.tast) list =
  List.map (fun (input,output) ->
    (Test_util.parse_ruby_string input,
     Typeinf.recognize_type_decls (Test_util.parse_ruby_string output)))
    [
      "a=0",
      "_typedecl(Object, :a); a=0";

      "if a then a=0 else a end",
      "_typedecl(Object, :a); if a then a=0 else a end";
      
      "while 0==0 do a=0 end",
      "_typedecl(Object, :a); while 0==0 do a=0 end";
      
      "a=a+a",
      "_typedecl(Object, :a); a=a+a";


      "a=a.m(a)",
      "_typedecl(Object, :a); a=a.m(a)";
      
      "a=a.m(a){ |b| c=b; a=c }",
      "_typedecl(Object, :a); \
       a=a.m(a){ |b| _typedecl(Object, :b, :c); (c=b; a=c) }";

      "def m() a; if 0==0 then a=1;a=a+1 end; end",
      "def m() _typedecl(Object, :a); (a; if 0==0 then a=1;a=a+1 end); end";

      (* self is not a local variable *)
      "def m() a=0; m(); end",
      "def m() _typedecl(Object, :a); (a=0; m()); end";

      (* constant is not a local variable *)
      "A_CONST=1; def m() x=A_CONST+1;x end; puts m()",
      "_typedecl(Object, :A_CONST);\
       (A_CONST=1; def m() _typedecl(Object, :x); (x=A_CONST+1;x) end; puts m())";
    ]

(* the following test cases need their expect values constructed
   manually because some strange insertion of blocks.  *)
let test_data2 : (Ast.tast * Ast.tast) list =
let open Ast.Untyped_constructor in
let open Ikra_type in
    List.map (fun (input,output) ->
      (Test_util.parse_ruby_string input,output))
      [
        "a=1;Array.new(10){|x| x*a }",
        type_decl (["a", Tobject], [
          block([
            local_assign("a", literal(Ast.Integer_literal 1));
            call (constant "Array", "new",
                  [literal (Ast.Integer_literal 10)],
                  Some(proc (var_pattern "x", (
                    type_decl (["x", Tobject], [
                      call (local_var "x", "*",
                            [local_var "a"], None)])))))])]);

        (* method parameters used in a block *)
        "def m(x) Array.new(1){|i| x+i} end; m(0)",
        block([
          method_def("m", array_pattern[var_pattern("x")], 
                     block([
                       call(constant("Array"),"new",
                            [literal(Ast.Integer_literal 1)],
	                    Some(proc(var_pattern("i"), (
                              type_decl([("i",Ikra_type.Tobject)], [
                                call(local_var("x"),"+",
                                     [local_var("i")],None)])))))]));
          call(local_var("**self**"),"m",[literal(Ast.Integer_literal 0)],None)]);

      "def m(a)  a end",
        method_def("m", array_pattern[var_pattern("a")], 
                     block([local_var "a"]));

      ]

let tests_let_intro = List.map (fun (input,output) ->
  ("let_intro(" ^ (Ast.string_of input) ^ ")")>::(fun _ ->
    assert_equal ~printer:(Ast.mlexp_of Ikra_type.string_of)
      output (Let_intro.let_intro input)))
  (test_data
   @ test_data2
  )


let suite =
  "suite">:::tests_let_intro

let () =
  run_test_tt_main suite

