module Constr :
  sig
    type t =
        Less_than of Ikra_type.t * Ikra_type.t
      | Equal of Ikra_type.t * Ikra_type.t
      (* Has_method(receiver,mname,proc) denotes that the receiver type
         has a method of mname whose argument and return types are denoted
         by proc.*)
      | Has_method of Ikra_type.t * string * Ikra_type.t
      | Defines_method of Ikra_type.t * string * Ikra_type.t

    (* smart constructors *)
    val less_than : Ikra_type.t * Ikra_type.t -> t
    val equal : Ikra_type.t * Ikra_type.t -> t
    val has_method : Ikra_type.t * string * Ikra_type.t -> t
    val defines_method : Ikra_type.t * string * Ikra_type.t -> t

    val type_order : Ikra_type.t -> int
    val compare_types : Ikra_type.t -> Ikra_type.t -> int
    val compare_type_list : Ikra_type.t list * Ikra_type.t list -> int
    val compare : t -> t -> int
    val cmp : t -> t -> bool
    val string_of : t -> string
  end

exception Contradiction of Constr.t

module Constraints :
  sig
    type elt = Constr.t
    type t = Set.Make(Constr).t
    val empty : t
    val is_empty : t -> bool
    val mem : elt -> t -> bool
    val add : elt -> t -> t
    val singleton : elt -> t
    val remove : elt -> t -> t
    val union : t -> t -> t
    val inter : t -> t -> t
    val diff : t -> t -> t
    val compare : t -> t -> int
    val equal : t -> t -> bool
    val subset : t -> t -> bool
    val iter : (elt -> unit) -> t -> unit
    val fold : (elt -> 'a -> 'a) -> t -> 'a -> 'a
    val for_all : (elt -> bool) -> t -> bool
    val exists : (elt -> bool) -> t -> bool
    val filter : (elt -> bool) -> t -> t
    val partition : (elt -> bool) -> t -> t * t
    val cardinal : t -> int
    val elements : t -> elt list
    val min_elt : t -> elt
    val max_elt : t -> elt
    val choose : t -> elt
    val split : elt -> t -> t * bool * t
  end
val set_print : Constraints.t -> string
val remove_method_defs : Constraints.t -> Constraints.t
