(* type embedding --- to convert between type annotated programs *)

open Ikra_type

let rec tast_of : Ikra_type.t -> Ast.tast =
  (* ast_of typ constructs an AST (i.e., Ruby embeddable)
     representation of typ *)

  let open Ast.Untyped_constructor in
      function
	| Tint -> constant("Integer")
	| Tfloat -> constant("Float")
	| Tbool -> constant("Bool")
	| Trange -> constant("Range")
	| Tobject -> constant("Object")
	| Tarray(n,t) ->
	  array [constant("Array");
		 literal(Ast.Integer_literal n);
		 tast_of t]
	| Tneighbor(m,n,t) ->
	  array [constant("Neigbor");
		 literal(Ast.Integer_literal m);
		 literal(Ast.Integer_literal n);
		 tast_of t]
	(* Though there's no chance to convert a type variable into
	   Ruby expression, the following is defined for
	   exhaustiveness. *)
	| Tvar(n) -> array[constant("TypeVar"); literal(Ast.Integer_literal n)]
	| Tclass(c) -> array[constant("Class"); literal(Ast.String_literal c)]
	| Ttuple(typs) -> array(constant "Tuple" :: List.map tast_of typs)

        (* TODO: fix this --- blk is ignored *)
	| Tproc(args,ret,blk) -> array(constant "Proc"
                                   :: tast_of ret :: List.map tast_of args)

(* let rec of_ast : Ast.tast -> t = *)
(*   (\* of_ast expression constructs an Ikra type representation for a *)
(*      type expressed as a Ruby expression*\) *)
(*   let open Ast in *)
(*       function *)
(* 	| Constant("Integer"),_ -> Tint *)
(* 	| Constant("Float"),_ -> Tfloat *)
(* 	| Constant(_),_ -> Tobject	(\* others are dynamic object *\) *)
(* 	| Array [Constant("Array"),_; *)
(* 		 Literal(Integer_literal n),_; *)
(* 		 t],_ -> Tarray(n, of_ast t) *)
(* 	| Array ((Constant("Tuple"),_)::element_types),_ -> *)
(*           Ttuple(List.map of_ast element_types) *)
(* 	| e -> print_string "checking e"; *)
(*                match e with (Array _,_) -> print_string "e is array"; *)
(*                raise (Invalid_argument *)
(* 			((Ast.string_of e) ^ *)
(* 			    " cannot be recognized as a type.")) *)

let is_ikra_type : Ast.tast -> bool =
  (* is_ikra_type exp determines if exp can be recognized as an Ikra
     type *)

  fun exp -> invalid_arg "foo"
    (* try let _ = of_ast exp in true *)
    (* with Invalid_argument _ -> false *)


let string_of : t -> string =
  fun texp -> let ast = tast_of texp in Ast.string_of ast

