# Conway's Game of Life
# usage:
#   [ikrac|ikrasim] game-of-life.rb [PATTERN [SIZE [MAX_GEN ["test"]]]]
#
#       PATTERN: the number from 0 to 3 specifying the initial pattern (0)
#       SIZE:    the size of the universe (512)
#       MAX_GEN: the maximum nember of generations (1000000)
#       "test":  runs in the test mode (not specified)
#                (no graphics/prints out the final universe)

def next_cell(live, count)
  if live==1 && count < 2
    0
  elsif live==1 && count < 4
    1
  elsif live==1
    0
  elsif count == 3
    1
  else
    0
  end
end


def init(size, height, width, bits)
  cr = (size-height)/2
  cc = (size-width)/2
  blen = height*width-1
  Array.new_(size,size){ |y,x|
    if cr <= y && y < cr+height &&
        cc <= x && x < cc+width
      p = blen - ( (y-cr)*width + (x-cc) )
      b = (bits & (1 << p)) >> p
      # puts "x=#{x} y=#{y} p =#{p} b=#{b}"
      b
    else
      0
    end
  }
end

def visualize(size, a)
  show(size, size, a.map{ |x| x * 0xffffff }.pack("I!*"))
  check_key()
end


def main(size,max_gen,universe, test)
  
  if !test
    visualize(size,universe)
  else                          # Since Ikra does not support if
    0                           # statements without an else branch,
  end                           # we need to write something here.
  #check(true)
  
  gen = 0
  while gen < max_gen
    universe = universe.neighbor9(0).map{ | n |
      count = n[-1,-1] + n[-1, 0] + n[-1, 1] +
              n[ 0,-1] +            n[ 0, 1] +
              n[ 1,-1] + n[ 1, 0] + n[ 1, 1]
      next_cell(n[0,0], count)
    }
    if ! test
      visualize(size,universe)
    else
      0
    end
    gen +=1
  end
  universe
end

def make_pattern(pattern, size)
  if pattern == 0
    init(size,4,6,0b101110100001101010010010) #Bunnies 11
# The Bunnies 11 pattern: http://www.conwaylife.com/wiki/Bunnies_11
# 101110
# 100001
# 101010
# 010010
#
# 4x6 = 24bits
# pattern = 101110100001101010010010
  elsif pattern == 1
    init(size,3,3,0b011110010)
  elsif pattern == 2
    init(size,3,7,0b010000000010001100111) #Acron
# The Acorn pattern: http://pmav.eu/stuff/javascript-game-of-life-v3.1.1
# 0100000  1
# 0001000  2
# 1100111  3
#
# 1234567
#
# 3x7 = 21bits 
# pattern = 010000000010001100111
  else
    init(size,5,5,0b1110110000000110110110101) #Callahan
  end

  # This pattern does not work as it is longer than 32 bits.
  # universe = init(5,8,0b0010011000010001001000001010010010101000) #23334M

end

# require 'optparse'

def m()
  pattern = 1
  size = 512
  generations = 1000000
  test = false
  if ARGV.length > 0
    pattern = ARGV.shift.to_i
  end
  if ARGV.length > 0
    size = ARGV.shift.to_i
  end
  if ARGV.length > 0
    generations = ARGV.shift.to_i
  end
  if ARGV.length > 0
    test = ARGV.shift == "test"
  end

  if ! test                     # do not load a graphics library for a
    load("./sdl-image.rb")      # test run, which should also be taken
                                # place on an environment without the
                                # graphics library.
  else
    0
  end

  u = main(size, generations, make_pattern(pattern,size), test)
  
  if test                                    # the pack/unpack
    puts u.pack("I!*").unpack("I!*").inspect # sequence converts an
                                             # array on GPU to a pure
                                             # Ruby array, which
                                             # provides the inspect
                                             # method.
  else
    0
  end
end

if __FILE__ == $0
  m()
end
