# Madelbrot set with animation

# set this to higher values and sleep well :)
#HM: less iterations for faster processing
#ITERATIONS = 200
ITERATIONS = 100

def encodeHSBcolor(h, s, b)
  # h - the hue component, where (h - floor(h))*360 produce the hue angle
  # s - the saturation of the color     (0.0 <= s <= 1.0)
  # b - the brightness of the color     (0.0 <= b <= 1.0)
  c = (1 - (2*b - 1).abs)*s
  h_ = (h - h.floor)*360/60
  x = c * (1 - (h_ % 2 - 1).abs)
  if    h_ < 1 then r1 = c; g1 = x; b1 = 0
  elsif h_ < 2 then r1 = x; g1 = c; b1 = 0
  elsif h_ < 3 then r1 = 0; g1 = c; b1 = x
  elsif h_ < 4 then r1 = 0; g1 = x; b1 = c
  elsif h_ < 5 then r1 = x; g1 = 0; b1 = c
  else              r1 = c; g1 = 0; b1 = x
  end
  m = b - c/2
  r = r1 + m; g = g1 + m; b = b1 + m
  (r*255).to_i * 0x10000 + (g*255).to_i * 0x100 + (b*255).to_i
end

def iterations(ar,ai)
  zr = ar
  zi = ai
  for i in 1..ITERATIONS
    zr0 = zr*zr - zi*zi + ar
    zi0 = 2 * zr*zi     + ai
    zr = zr0
    zi = zi0
    if cabs_square(zr,zi) >= 2*2
      return i
    end
  end
  ITERATIONS
end

def cabs_square(zr,zi)
  zr*zr + zi*zi
end

def mandelbrot(width,height,x_start, y_start, step_x, step_y)
  Array.new_(height,width){ |y_pixel, x_pixel|
    y = y_pixel*step_y+y_start
    x = x_pixel*step_x+x_start
    c = iterations(x,y)
  }
end

def iter2bitmap(iter_map)
  iter_map.map{ |c| 
    c==ITERATIONS ? 0: encodeHSBcolor(c*1.0/ITERATIONS,1.0,0.5)
  }
end

def main(width,height,x_start,x_end,y_start,y_end, not_test)
  if not_test
    load("./sdl-image.rb")
  end
  
  startTime = Time.new
  
  frames = 0
  
  while true
    step_x = (x_end - x_start) / width
    step_y = (y_end - y_start) / height
    
    m = mandelbrot(width,height,x_start, y_start, step_x, step_y)
    if not_test
      show(width,height,iter2bitmap(m).pack("I!*"))
      check_key
      
      x_start += (x_end-x_start)*0.01
      x_end   -= (x_end-x_start)*0.01
      y_start += (y_end-y_start)*0.01
      y_end   -= (y_end-y_start)*0.01
      
      frames+=1
      if frames%10 == 0
        print frames/((Time.new - startTime).to_f)
        puts " fps"
      end
    else
      puts m.pack("I!*").unpack("I!*").inspect
      return
    end
  end
end

if __FILE__ == $0
  # this is the interesting area of the complex plane
  x_start = -2.0;  x_end = 0.5
  y_start = -1.05; y_end = 0.95
  width = 192*2
  height = 150*2

  if loop = ARGV[0]!="test"
    0
  else
    ARGV.shift
    width=height=ARGV.shift.to_i
  end
  main(width, height, x_start,x_end,y_start, y_end, loop)
end
