#!/usr/bin/env ruby
# create an archive for distribution
require 'tmpdir'

def main(branch_name)
  
  # This script is $src_root/util/make-archive.rb, hence the root
  # directory is:
  src_root=File.expand_path(File.dirname(__FILE__),"..")
  
  
  Dir.mktmpdir do |d|
    # from the local repository, create a clean checkout of the
    # specified branch
    system("git clone --branch #{branch_name} --local  #{src_root} #{d}") or
      raise "failed to clone"
    
    Dir.chdir d
    
    # git describe --tags will print the latest tag, which should be
    # something like "v.0.1.99":
    tag_name=`git describe --tags`.chomp
    
    # clean everything
    #Dir.chdir(src_root)
    #system("make clean") or raise "failed to clean"
    
    # create version.ml
    system("cd src; make version.ml") or raise "failed to create version.ml"
    
    # make archive
    virtual_dir_name ="ikra-#{tag_name}"
    archive_file_name="../#{virtual_dir_name}.tgz"
    archive_command = ["tar",
                       "--create",
                       "--gzip",
                       "--file", archive_file_name,
                       "--exclude-vcs", "--exclude-backups",
                       "--exclude='*~'", "--exclude=./bin",
                       "--exclude='*/ikra.gen/*'",
                       # rename the source root directory name
                       "--transform", "'s/^\\./#{virtual_dir_name}/'" ,
                       # the directory to archive
                       "."
                      ].join(" ")
    puts archive_command
    system(archive_command) or raise "failed to archive:\n#{archive_command}"
  end
  
end

if __FILE__ == $0
  main(*ARGV)
end
